!==============================================================================
! LAGRANGE-Polynomials for GAUSS-LOBATTO collocation points
!
! n=degree of the LAGRANGE polynomials (greater than 1)
! i=index of the LAGRANGE polynomial (between 0 and n)
!==============================================================================

subroutine lgll(n,i,x,y)
implicit none

	!======================================================================
	! variables
	!======================================================================
	
	integer, intent(in) :: n, i
	real, intent(in) :: x
	real, intent(out) :: y
	
	integer :: k
	real, dimension(0:7) :: knots
	
	y=1.0
	
	!======================================================================
	! determine collocation points (knots)
	!======================================================================
	
	if (n==2) then
		
		knots(0)=-1.0
		knots(1)=0.0
		knots(2)=1.0
		
	elseif (n==3) then
		
		knots(0)=-1.0
		knots(1)=-0.4472135954999579
		knots(2)=0.4472135954999579
		knots(3)=1.0
		
	elseif (n==4) then
		
		knots(0)=-1.0
		knots(1)=-0.6546536707079772
		knots(2)=0.0
		knots(3)=0.6546536707079772
		knots(4)=1.0
		
	elseif (n==5) then
		
		knots(0)=-1.0
		knots(1)=-0.7650553239294647
		knots(2)=-0.2852315164806451
		knots(3)=0.2852315164806451
		knots(4)=0.7650553239294647
		knots(5)=1.0
		
	elseif (n==6) then
		
		knots(0)=-1.0
		knots(1)=-0.8302238962785670
		knots(2)=-0.4688487934707142
		knots(3)=0.0
		knots(4)=0.4688487934707142
		knots(5)=0.8302238962785670
		knots(6)=1.0
		
	elseif (n==7) then
		
		knots(0)=-1.0
		knots(1)=-0.8717401485096066
		knots(2)=-0.5917001814331423
		knots(3)=-0.2092992179024789
		knots(4)=0.2092992179024789
		knots(5)=0.5917001814331423
		knots(6)=0.8717401485096066
		knots(7)=1.0
		
	endif
	
	!======================================================================
	! compute value of the LAGRANGE polynomial
	!======================================================================
	
	do k=0,n
	
		if (k /= i) then
	
			y=y*(x-knots(k))/(knots(i)-knots(k))
		
		endif
		
	enddo
	

end subroutine lgll


!==============================================================================
! derivatives of LAGRANGE-GAUSS-LOBATTO polynomials at the collocation points
! n=degree of the LAGRANGE polynomial
! i=index of the LAGRANGE polynomial
! j=index of the collocation point where the derivative is evaluated
! y=output=derivative of the LAGRANGE polynomial at the collocation point j
!==============================================================================

subroutine dlgll(n,i,j,y)
implicit none

	!======================================================================
	! local variables
	!======================================================================
	
	integer, intent(in) :: n, i, j
	real, intent(out) :: y
	
	integer :: k
	real, dimension(0:7) :: knots
	
	!======================================================================
	! determine collocation points (knots)
	!======================================================================
	
	if (n==2) then
		
		knots(0)=-1.0
		knots(1)=0.0
		knots(2)=1.0
		
	elseif (n==3) then
		
		knots(0)=-1.0
		knots(1)=-0.4472135954999579
		knots(2)=0.4472135954999579
		knots(3)=1.0
		
	elseif (n==4) then
		
		knots(0)=-1.0
		knots(1)=-0.6546536707079772
		knots(2)=0.0
		knots(3)=0.6546536707079772
		knots(4)=1.0
		
	elseif (n==5) then
		
		knots(0)=-1.0
		knots(1)=-0.7650553239294647
		knots(2)=-0.2852315164806451
		knots(3)=0.2852315164806451
		knots(4)=0.7650553239294647
		knots(5)=1.0
		
	elseif (n==6) then
		
		knots(0)=-1.0
		knots(1)=-0.8302238962785670
		knots(2)=-0.4688487934707142
		knots(3)=0.0
		knots(4)=0.4688487934707142
		knots(5)=0.8302238962785670
		knots(6)=1.0
		
	elseif (n==7) then
		
		knots(0)=-1.0
		knots(1)=-0.8717401485096066
		knots(2)=-0.5917001814331423
		knots(3)=-0.2092992179024789
		knots(4)=0.2092992179024789
		knots(5)=0.5917001814331423
		knots(6)=0.8717401485096066
		knots(7)=1.0
		
	endif
	
	!======================================================================
	! compute derivative of LAGRANGE polynomial i at collocation point j
	!======================================================================
	
	if (i==j) then
		
		y=0.0
	
		do k=0,n
		
			if (k /= i) then
				
				y=y+1.0/(knots(i)-knots(k))
			
			endif
			
		enddo
		
	else
		
		y=1.0/(knots(i)-knots(j))
	
		do k=0,n
		
			if ((k /= i) .and. (k /= j)) then
				
				y=y*(knots(j)-knots(k))/(knots(i)-knots(k))
			
			endif
			
		enddo
		
	endif

end subroutine dlgll
