!*****************************************************************************
!***************** parameters and global variables ***************************
!*****************************************************************************

!=============================================================================
! Predefined parameters
!=============================================================================

module parameters
implicit none

	integer, parameter :: nx_max=10		! maximum number of elements in x direction
	integer, parameter :: ny_max=10		! maximum number of elements in y direction
	integer, parameter :: nz_max=20		! maximum number of elements in z direction
	
	integer, parameter :: lpd=4		! LAGRANGE polynomial degree
	
	integer, parameter :: maxnt=15000	! maximum number of time steps
	integer, parameter :: maxnr=800         ! maximum number of receivers
        integer, parameter :: pml=3
	
        integer, parameter :: nrdiss=2          ! number of relaxation mechanisms

	real, parameter :: pi=3.1415926535898

end module parameters
    
!==============================================================================
! global variables
!==============================================================================

module variables
use parameters
implicit none

        integer :: itmem, run_the_programme
        real :: ispml
        real :: E_kin_total, E_kin_total_old
        
	!======================================================================
	! model dimensions
	!======================================================================

	integer :: nx, ny, nz			! number of elements in the respective coordinate directions
	integer :: ix_multi, iy_multi, iz_multi	! multi index of the elements

	integer :: px, py, pz			! number of processors in the respective coordinate directions
	integer :: index_x, index_y, index_z
	integer :: p, my_rank, ierr

	!======================================================================
	! physical model parameters
	!======================================================================
	
	real, dimension(0:nx_max,0:ny_max,0:nz_max,0:lpd,0:lpd,0:lpd) :: rhoinv, mu, lambda, kappa, mu_tau
	real, dimension(0:nx_max,0:ny_max,0:nz_max,0:lpd,0:lpd,0:lpd) :: A, B, C
	real, dimension(0:nx_max,0:ny_max,0:nz_max,0:lpd,0:lpd,0:lpd) :: tau
	real, dimension(1:nrdiss,0:nx_max,0:ny_max,0:nz_max,0:lpd,0:lpd,0:lpd) :: tau_sp
	
	!======================================================================
	! displacement fields, strain fields, mass matrix and memory variables
	!======================================================================
	
	real, dimension(0:nx_max,0:ny_max,0:nz_max,0:lpd,0:lpd,0:lpd) :: ux, uy, uz
	real, dimension(0:nx_max,0:ny_max,0:nz_max,0:lpd,0:lpd,0:lpd) :: vx, vy, vz

        real, dimension(0:nx_max,0:ny_max,0:nz_max,0:lpd,0:lpd,0:lpd) :: MM, preco, maxfield

        real, dimension(0:nx_max,0:ny_max,0:nz_max,0:lpd,0:lpd,0:lpd) :: exx, eyy, ezz
        real, dimension(0:nx_max,0:ny_max,0:nz_max,0:lpd,0:lpd,0:lpd) :: exy, exz, eyz
        real, dimension(0:nx_max,0:ny_max,0:nz_max,0:lpd,0:lpd,0:lpd) :: eyx, ezx, ezy

        real, dimension(0:nx_max,0:ny_max,0:nz_max,0:lpd,0:lpd,0:lpd) :: dxux, dyux, dzux
        real, dimension(0:nx_max,0:ny_max,0:nz_max,0:lpd,0:lpd,0:lpd) :: dxuy, dyuy, dzuy
        real, dimension(0:nx_max,0:ny_max,0:nz_max,0:lpd,0:lpd,0:lpd) :: dxuz, dyuz, dzuz

        real, dimension(1:nrdiss,0:nx_max,0:ny_max,0:nz_max,0:lpd,0:lpd,0:lpd) :: Mxx, Myy, Mzz
        real, dimension(1:nrdiss,0:nx_max,0:ny_max,0:nz_max,0:lpd,0:lpd,0:lpd) :: Mxy, Mxz, Myz
	
	!======================================================================
	! weak form stress fields
	!======================================================================
	
	real, dimension(0:nx_max,0:ny_max,0:nz_max,0:lpd,0:lpd,0:lpd) :: sx, sy, sz
	
	!======================================================================
	! strong form stress fields
	!======================================================================
	
	real, dimension(0:nx_max,0:ny_max,0:nz_max,0:lpd,0:lpd,0:lpd) :: sxx,syy,szz
	real, dimension(0:nx_max,0:ny_max,0:nz_max,0:lpd,0:lpd,0:lpd) :: sxx_g,syy_g,szz_g
	real, dimension(0:nx_max,0:ny_max,0:nz_max,0:lpd,0:lpd,0:lpd) :: sxy,sxz,syz

        real, dimension(0:nx_max,0:ny_max,0:nz_max,0:lpd,0:lpd,0:lpd) :: sxx_pml,syy_pml,szz_pml
        real, dimension(0:nx_max,0:ny_max,0:nz_max,0:lpd,0:lpd,0:lpd) :: sxy_pml,syz_pml,sxz_pml
        real, dimension(0:nx_max,0:ny_max,0:nz_max,0:lpd,0:lpd,0:lpd) :: syx_pml,szy_pml,szx_pml

        !======================================================================
        ! PML parameters
        !======================================================================

        real, dimension(0:nx_max,0:ny_max,0:nz_max,0:lpd,0:lpd,0:lpd) :: prof_x, prof_y, prof_z, prof, taper
        integer, dimension(1:2*pml) :: pmlidx, pmlidy, pmlidz
        real :: E_kin, E_kin_old
	
	!======================================================================
	! geometrical parameters
	!======================================================================
	
	real, dimension(0:7) :: knots, w
	real, dimension(0:7,0:7) :: dl
	
	real :: dx, dy, dz
	real :: xmin,xmax,ymin,ymax,zmin,zmax
	real :: xmin_global, xmax_global, ymin_global, ymax_global, zmin_global, zmax_global
	
	real, dimension(0:nx_max,0:lpd) :: x
	real, dimension(0:ny_max,0:lpd) :: y
	real, dimension(0:nz_max,0:lpd) :: z
	
	real, dimension(0:nx_max,0:ny_max,0:nz_max,0:lpd,0:lpd,0:lpd) :: sin_theta, r, cot_theta, cos_theta

	real :: Jac
	
	!======================================================================
	! time parameters
	!======================================================================
	
	integer :: it, nt
	real :: dt

	!======================================================================
	! Source Variables
	!======================================================================

	integer :: source_type
	integer :: isx, isy, isz, isx_n, isy_n, isz_n
        
	real :: xxs, yys, zzs
	real :: is_source, source_processor
	real, dimension(1:maxnt) :: so
	
        real :: MOM_xx, MOM_yy, MOM_zz, MOM_xy, MOM_xz, MOM_yz

        real, dimension(0:nx_max,0:ny_max,0:nz_max,0:lpd,0:lpd,0:lpd) :: sf_dx,sf_dy,sf_dz
        
	!======================================================================
	! receiver variables
	!======================================================================
	
        integer :: nr_global, nr                   			                        	! number of receivers (first line of recfile)
	real :: recloc_global(1:3,1:maxnr), recloc(1:3,1:maxnr), recloc_std(1:3,1:maxnr)     		! receiver location x, y, depth
        integer :: rx(1:maxnr), ry(1:maxnr), rz(1:maxnr)

	character(len=6) :: station_name_global(1:maxnr), station_name_local(1:maxnr)

        !======================================================================
        ! output
        !======================================================================
        
        character(len=100) :: ofd                                                                       ! output file directory
        character(len=100) :: ffd                                                                       ! forward field directory
        
        real :: seismogram_x(1:maxnr,1:maxnt)
        real :: seismogram_y(1:maxnr,1:maxnt)
        real :: seismogram_z(1:maxnr,1:maxnt)
	
	!======================================================================
	! output variables
	!======================================================================
	
	integer :: ssamp, iplanex, iplaney, iplanez
	
	!======================================================================
	! other variables
	!======================================================================
	
	integer :: is_aniso, is_diss, is_formatted

        !======================================================================
        ! adjoint computations
        !======================================================================

        integer :: samp_ad
        integer :: saving_vector(1:maxnt)
	

end module variables
!==============================================================================
